import pytest
from ewoksnotify.tasks.tone import ToneTask


def test_ToneTask(tmp_path):
    """test ToneTask function"""
    task = ToneTask(
        inputs={
            "object": 12,
        },
    )
    task.run()
    assert task.outputs.object == 12

    task = ToneTask(
        inputs={
            "object": "toto",
            "tone_type": "info",
            "theme": "chime",
        }
    )
    task.run()
    assert task.outputs.object == "toto"

    with pytest.raises(ValueError):
        # check raise an error if path not part of PROCESSED_DATA
        ToneTask(
            inputs={
                "object": "toto",
                "tone_type": "test",
            }
        ).run()

    with pytest.raises(ValueError):
        # check raise an error if path not part of PROCESSED_DATA
        ToneTask(
            inputs={
                "object": "toto",
                "theme": "test",
            }
        ).run()



import smtplib
from ewokscore.task import Task


class EmailTask(
    Task,
    input_names=("subject", "from_addr", "to_addrs", "text"),  # type: ignore
    optional_input_names=("mail_options", "rcpt_options", "host", "port"),  # type: ignore
):
    """
    Generic task to send a simple email
    Uses 'smtps.esrf.fr' so will only work inside esrf
    """

    def run(self):
        port = self.get_input_value("port", 0)
        assert isinstance(port, int), "port is expected to be an int"
        host = self.get_input_value("host", "smtps.esrf.fr")
        server = smtplib.SMTP(host, port, timeout=5)
        server.sendmail(
            self.inputs.from_addr,
            self.inputs.to_addrs,
            f"Subject: {self.inputs.subject}\n\n{self.inputs.text}",
        )
        server.quit()



import os
import pytest
from ewoksnotify.tasks.icat import PublishProcessedDataFolderTask


def test_PublishProcessedDataFolderTask(tmp_path):
    """test PublishProcessedDataFolderTask function"""
    inputs = {
        "beamline": "my_beamline",
        "proposal": "my_proposal",
        "dataset": "my_dataset",
        "path": tmp_path,
        "raw": "/path/to/raw/dataset",
        "dry_run": True,
    }
    task = PublishProcessedDataFolderTask(
        inputs=inputs,
    )

    # check raise an error if path doesn't exists
    inputs["path"] = os.path.join(tmp_path, "PROCESSED_DATA", "dataset")
    task = PublishProcessedDataFolderTask(
        inputs=inputs,
    )
    with pytest.raises(ValueError):
        task.run()
    os.makedirs(inputs["path"])
    task.run()



import os
import logging
from ewokscore.task import Task

try:
    from pyicat_plus.client.main import IcatClient  # noqa F401
except ImportError:
    has_pyicat_plus = False
else:
    has_pyicat_plus = True

_logger = logging.getLogger(__name__)


class PublishProcessedDataFolderTask(
    Task,
    input_names=(  # type: ignore
        "beamline",
        "proposal",
        "dataset",
        "path",
        "raw",
    ),
    optional_input_names=(  # type: ignore
        "metadata",
        "dry_run",
    ),
):
    """publish a folder containing 'processed data' to icat for the provided beamline && dataset && proposal"""

    def run(self):
        beamline = self.inputs.beamline
        proposal = self.inputs.proposal
        dataset = self.inputs.dataset
        raw = self.inputs.raw
        metadata = self.get_input_value("metadata", {})
        path = self.inputs.path
        dry_run = self.get_input_value("dry_run", False)

        # checks (need because can be pass by a GUI and will set those values to empty string mostly)
        missing = []
        if beamline in (None, ""):
            missing.append("beamline")
        if proposal in (None, ""):
            missing.append("proposal")
        if dataset in (None, ""):
            missing.append("dataset")
        if raw is None or len(raw) == 0:
            missing.append("raw")

        if len(missing) > 0:
            mess = f"Missing information about {','.join(missing)}"
            _logger.error(mess)
            raise ValueError(mess)
        if not os.path.exists(path):
            raise ValueError(f"path: {path} doesn't exists")

        # publish
        if not has_pyicat_plus:
            raise ImportError("pyicat_plus not installed")

        if not dry_run:
            icat_client = IcatClient(
                metadata_urls=("bcu-mq-01.esrf.fr:61613", "bcu-mq-02.esrf.fr:61613")
            )

            icat_client.store_processed_data(
                beamline=beamline,
                proposal=proposal,
                dataset=dataset,
                path=path,
                metadata=metadata,
                raw=raw,
            )



from ewoksnotify.gui.tone import ToneSelection, ToneWindow
from silx.gui.utils.testutils import SignalListener


def test_ToneSelection(qtapp):
    """test the ToneSelection widget"""
    widget = ToneSelection()
    listener = SignalListener()
    widget.sigConfigChanged.connect(listener)
    assert widget.getConfiguration() == {
        "theme": "zelda",
        "tone_type": "success",
    }

    assert listener.callCount() == 0

    new_config = {
        "theme": "chime",
        "tone_type": "error",
    }
    widget.setConfiguration(new_config)
    assert widget.getConfiguration() == new_config
    assert listener.callCount() == 2


def test_ToneWindow(qtapp):
    """test the ToneWindow"""
    widget = ToneWindow()
    assert widget.getConfiguration() == {
        "theme": "zelda",
        "tone_type": "success",
        "muted": False,
    }

    new_config = {
        "theme": "chime",
        "tone_type": "error",
        "muted": True,
    }
    widget.setConfiguration(new_config)
    assert widget.getConfiguration() == new_config



from orangewidget import gui
from silx.gui import qt
from ewoksorange.bindings.owwidgets import OWEwoksWidgetOneThreadPerRun
from ewoksnotify.tasks.waiter import WaiterTask


class WaiterOW(
    OWEwoksWidgetOneThreadPerRun,
    ewokstaskclass=WaiterTask,  # type: ignore
):
    """
    simple widget that sleeps for n seconds when received an object then release it
    """

    name = "waiter"
    id = "orangecontrib.ewoksnotify.WaiterOW"
    description = "simple widget that sleeps for n seconds when received an object then release it"
    icon = "icons/waiter.png"
    priority = 160
    keywords = ["waiter", "ewoks", "ewoksnotify"]

    _ewoks_inputs_to_hide_from_orange = ("seconds",)

    want_main_area = True
    want_control_area = False
    resizing_enabled = False

    def __init__(self, parent=None):
        super().__init__(parent)
        self.pop_up = None
        layout = gui.hBox(self.mainArea, self.name).layout()
        # sleeping time
        layout.addWidget(qt.QLabel("waiting time", self))
        self._secondQSB = qt.QSpinBox(self)
        self._secondQSB.setValue(3)
        self._secondQSB.setRange(1, 2147483647)
        self._secondQSB.setSuffix(" seconds")
        layout.addWidget(self._secondQSB)

    def get_task_inputs(self):
        task_inputs = super().get_task_inputs()
        # when we are with the gui we go for the Qt pop up instead of the message to stdout
        task_inputs.update(
            {
                "seconds": float(self._secondQSB.value()),
            }
        )
        return task_inputs



from silx.gui import qt
from ewoksnotify.tasks.tone import has_chime
from ewoksnotify.gui.actions import ParametersAction

if has_chime:
    import chime


class ToneSelection(qt.QWidget):
    """
    Widget to select tone type and theme
    """

    sigConfigChanged = qt.Signal()
    """emit when the configuration is changed"""

    def __init__(self, parent=None) -> None:
        super().__init__(parent)
        if not has_chime:
            raise ImportError("chime not available")
        self.setLayout(qt.QFormLayout())

        # define theme
        self._theme = qt.QComboBox(self)
        self._theme.addItems(chime.themes())
        self.layout().addRow("Theme", self._theme)

        # define type
        self._toneType = qt.QComboBox(self)
        self._toneType.addItems(("success", "warning", "error", "info"))
        self.layout().addRow("Tome type", self._toneType)

        # set up
        self.setTheme("zelda")
        self.setToneType("success")

        # connect signal / slot
        self._theme.currentIndexChanged.connect(self._changed)
        self._toneType.currentIndexChanged.connect(self._changed)

    def _changed(self):
        self.sigConfigChanged.emit()

    def getToneType(self) -> str:
        return self._toneType.currentText()

    def setToneType(self, tone_type: str) -> None:
        self._toneType.setCurrentText(tone_type)

    def getTheme(self) -> str:
        return self._theme.currentText()

    def setTheme(self, theme: str) -> None:
        self._theme.setCurrentText(theme)

    def getConfiguration(self):
        return {
            "tone_type": self.getToneType(),
            "theme": self.getTheme(),
        }

    def setConfiguration(self, config: dict):
        tone_type = config.get("tone_type", None)
        if tone_type is not None:
            self.setToneType(tone_type)
        tone_theme = config.get("theme", None)
        if tone_theme is not None:
            self.setTheme(tone_theme)


class ToneSelectionDialog(qt.QDialog):
    """
    Dialog dedicated to tone selection
    """

    def __init__(self, parent=None) -> None:
        super().__init__(parent)
        self.setLayout(qt.QVBoxLayout())
        self._mainWidget = ToneSelection()
        self.layout().addWidget(self._mainWidget)

        types = qt.QDialogButtonBox.Ok | qt.QDialogButtonBox.Cancel
        self._buttons = qt.QDialogButtonBox(parent=self)
        self._buttons.setStandardButtons(types)
        self.layout().addWidget(self._buttons)

        # connect signal / slot
        self._buttons.accepted.connect(self.accept)
        self._buttons.rejected.connect(self.reject)

    # expose API
    def getToneType(self):
        return self._mainWidget.getToneType()

    def setToneType(self, *args, **kwargs):
        self._mainWidget.setToneType(*args, **kwargs)

    def getTheme(self):
        return self._mainWidget.getTheme()

    def setTheme(self, *args, **kwargs):
        self._mainWidget.setTheme(*args, **kwargs)


class ToneWindow(qt.QMainWindow):
    """Main window use to define tune and turn on / off the sound"""

    DEFAULT_CONFIGURATION = {
        "tone_type": "success",
        "theme": "zelda",
        "muted": False,
    }

    def __init__(self, parent=None):
        super().__init__(parent)
        self._configuration = self.DEFAULT_CONFIGURATION
        self._soundButton = qt.QPushButton(parent=self)
        self._soundButton.setMinimumSize(150, 100)
        self._soundButton.setCheckable(True)
        self.setCentralWidget(self._soundButton)

        # add toolbar
        toolbar = qt.QToolBar(self)
        toolbar.setMovable(False)
        toolbar.setFloatable(False)
        self.addToolBar(qt.Qt.TopToolBarArea, toolbar)

        # add Parameters Action
        self._parametersAction = ParametersAction(toolbar)
        toolbar.addAction(self._parametersAction)
        # set up
        self._updateButtonIcon()

        # connect signal / slot
        self._soundButton.toggled.connect(self._switchMute)
        self._parametersAction.triggered.connect(self._parametersTriggered)

    def _parametersTriggered(self):
        dialog = ToneSelectionDialog()
        dialog.setToneType(self.getConfiguration()["tone_type"])
        dialog.setTheme(self.getConfiguration()["theme"])
        if dialog.exec_():
            self._configuration["tone_type"] = dialog.getToneType()
            self._configuration["theme"] = dialog.getTheme()

    def _switchMute(self):
        self._configuration["muted"] = not self._configuration["muted"]
        self._updateButtonIcon()

    def _updateButtonIcon(self):
        style = qt.QApplication.style()
        if self._configuration["muted"]:
            icon = style.standardIcon(qt.QStyle.SP_MediaVolumeMuted)
        else:
            icon = style.standardIcon(qt.QStyle.SP_MediaVolume)
        self._soundButton.setIcon(icon)

    def getConfiguration(self) -> dict:
        return self._configuration

    def setConfiguration(self, config: dict):
        self._configuration = config



import sys

try:
    import chime
except ImportError:
    has_chime = False
else:
    has_chime = True
from ewokscore.task import Task


class ToneTask(
    Task,
    input_names=("object",),  # type: ignore
    optional_input_names=("tone_type", "theme", "muted", "with_text_message"),  # type: ignore
    output_names=("object",),  # type: ignore
):
    """publish a folder containing 'processed data' to icat for the provided beamline && dataset && proposal"""

    def run(self):
        muted = self.get_input_value("muted", False)
        tone_theme = self.get_input_value("theme", "zelda")
        if tone_theme not in chime.themes():
            raise ValueError(f"invalid theme requested: {tone_theme}")

        chime.theme(tone_theme)

        if not muted:
            tone_type = self.get_input_value("tone_type", "success")
            if tone_type == "success":
                chime.success()
            elif tone_type == "warning":
                chime.warning()
            elif tone_type == "error":
                chime.error()
            elif tone_type == "info":
                chime.info()
            else:
                raise ValueError(f"tone type ({tone_type}) is not handled")

        if self.get_input_value("with_text_message", False):
            sys.stdout.write(f"{object} received")
        # providing an output is discussable. But user are not force to connect it to anything
        self.outputs.object = self.inputs.object



from typing import Optional
from silx.gui import qt
from silx.gui import icons


class EmailComposition(qt.QWidget):
    """
    Widget used to compose an email

    Please see https://confluence.esrf.fr/display/SCKB/Rules+about+email for email rules at esrf
    """

    sigChanged = qt.Signal()
    """emit when the composition changed"""

    def __init__(self, parent: Optional[qt.QWidget]) -> None:
        super().__init__(parent)
        self.setLayout(qt.QFormLayout())
        # from
        self._fromAddressesQLE = qt.QLineEdit(self)
        singleEmailValidator = qt.QRegularExpressionValidator(
            qt.QRegularExpression(
                r"(^[a-zA-Z0-9_.+-]+@[a-zA-Z0-9-]+\.[a-zA-Z0-9-.]+$)"
            ),
            self._fromAddressesQLE,
        )
        self._fromAddressesQLE.setValidator(singleEmailValidator)
        self.layout().addRow("from", self._fromAddressesQLE)
        # to
        self._toAdressesQLE = qt.QLineEdit(self)
        severalEmailValidator = qt.QRegularExpressionValidator(
            qt.QRegularExpression(
                r"(([a-zA-Z0-9_\-\.]+)@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.)|(([a-zA-Z0-9\-]+\.)+))([a-zA-Z]{2,4}|[0-9]{1,3})(\]?)(\s*;\s*|\s*$))*"
            ),
            self,
        )
        self._toAdressesQLE.setValidator(severalEmailValidator)
        self.layout().addRow("to", self._toAdressesQLE)
        # subject
        self._subjectQLE = qt.QLineEdit("ewoks notification", self)
        self.layout().addRow("subject", self._subjectQLE)
        # text
        self._textQLE = qt.QPlainTextEdit("")
        self._textQLE.setPlaceholderText("message")
        self.layout().addRow(self._textQLE)

        # connect signal / slot
        self._fromAddressesQLE.editingFinished.connect(self.sigChanged)
        self._toAdressesQLE.editingFinished.connect(self.sigChanged)
        self._subjectQLE.editingFinished.connect(self.sigChanged)
        self._textQLE.textChanged.connect(self.sigChanged)

    def getFromAddr(self) -> str:
        return self._fromAddressesQLE.text()

    def setFromAddr(self, from_addr: str):
        self._fromAddressesQLE.setText(str(from_addr))

    def getToAddrs(self) -> tuple:
        adresses = self._toAdressesQLE.text().replace(" ", "")
        adresses.replace(",", ";")
        return tuple(set(adresses.split(";")))

    def setToAddrs(self, to_addrs) -> None:
        if not isinstance(to_addrs, str):
            to_addrs = ";".join(to_addrs)
        self._toAdressesQLE.setText(str(to_addrs))

    def getText(self) -> str:
        return self._textQLE.toPlainText()

    def setText(self, text: str) -> None:
        self._textQLE.setPlainText(text)

    def getSubject(self) -> str:
        return self._subjectQLE.text()

    def setSubject(self, subject: str):
        self._subjectQLE.setText(str(subject))

    def getConfiguration(self) -> dict:
        return {
            "subject": self.getSubject(),
            "from_addr": self.getFromAddr(),
            "to_addrs": self.getToAddrs(),
            "text": self.getText(),
        }

    def setConfiguration(self, config: dict):
        if not isinstance(config, dict):
            raise TypeError
        subject = config.get("subject", None)
        if subject is not None:
            self.setSubject(str(subject))
        from_addr = config.get("from_addr", None)
        if from_addr is not None:
            self.setFromAddr(from_addr=str(from_addr))
        to_addrs = config.get("to_addrs", None)
        if to_addrs is not None:
            self.setToAddrs(to_addrs=to_addrs)
        text = config.get("text", None)
        if text is not None:
            self.setText(str(text))


class EmailSettings(qt.QWidget):
    """
    Widget to set up mail server
    """

    sigChanged = qt.Signal()
    """emit when the settings changed"""

    def __init__(self, parent: Optional[qt.QWidget]) -> None:
        super().__init__(parent)
        self.setLayout(qt.QFormLayout(self))
        # server
        self._hostQLE = qt.QLineEdit("smtps.esrf.fr", self)
        self.layout().addRow("host", self._hostQLE)
        # port
        self._portQLE = qt.QLineEdit("0", self)
        self._portQLE.setValidator(qt.QIntValidator(self._portQLE))
        self.layout().addRow("port", self._portQLE)

        # connect signal / slot
        self._hostQLE.editingFinished.connect(self.sigChanged)
        self._portQLE.editingFinished.connect(self.sigChanged)

    def getConfiguration(self) -> dict:
        return {"host": self._hostQLE.text(), "port": int(self._portQLE.text())}

    def setConfiguration(self, config: dict):
        server = config.get("host", None)
        if server is not None:
            self._hostQLE.setText(str(server))
        port = config.get("port", None)
        if port is not None:
            self._portQLE.setText(str(port))


class EmailWidget(qt.QTabWidget):
    """
    Main widget to send email
    """

    sigChanged = qt.Signal()
    """emit when the email settings or composition changed"""

    def __init__(self, parent: Optional[qt.QWidget] = None) -> None:
        super().__init__(parent)

        self.setTabPosition(qt.QTabWidget.East)

        self._compositionWidget = EmailComposition(self)
        composeIcon = icons.getQIcon("ewoksnotify:gui/icons/compose")
        self.addTab(
            self._compositionWidget,
            composeIcon,
            "",
        )

        self._serverSettingsWidget = EmailSettings(self)
        settingsIcon = icons.getQIcon("ewoksnotify:gui/icons/parameters")
        self.addTab(
            self._serverSettingsWidget,
            settingsIcon,
            "",
        )

        # connect signal / slot
        self._compositionWidget.sigChanged.connect(self.sigChanged)
        self._serverSettingsWidget.sigChanged.connect(self.sigChanged)

    def getConfiguration(self) -> dict:
        config = self._serverSettingsWidget.getConfiguration()
        config.update(self._compositionWidget.getConfiguration())
        return config

    def setConfiguration(self, config: dict):
        self._serverSettingsWidget.setConfiguration(config)
        self._compositionWidget.setConfiguration(config)



import time
from ewokscore.task import Task


class WaiterTask(
    Task,
    input_names=("object", "seconds"),  # type: ignore
    output_names=("object",),  # type: ignore
):
    """
    Simple task which waits for n seconds and release the object
    """

    def run(self):
        time.sleep(self.inputs.seconds)
        self.outputs.object = self.inputs.object



from silx.gui import qt
from silx.gui import icons


class PadlockButton(qt.QPushButton):
    """Simple button to define a button with PadLock icons"""

    sigLockChanged = qt.Signal(bool)
    """signal emitted when the lock status change"""

    def __init__(self, parent):
        qt.QPushButton.__init__(self, parent)
        self._lockIcon = icons.getQIcon("ewoksnotify:gui/icons/locked")
        self._unlockIcon = icons.getQIcon("ewoksnotify:gui/icons/unlocked")
        self.setIcon(self._unlockIcon)
        self.setCheckable(True)

        # connect signals
        self.toggled.connect(self._updateDisplay)

    def setLock(self, lock: bool):
        self.setChecked(lock)
        self._updateDisplay(lock)

    def _updateDisplay(self, checked):
        _icon = self._lockIcon if checked else self._unlockIcon
        self.setIcon(_icon)
        self.sigLockChanged.emit(checked)

    def isLocked(self) -> bool:
        return self.isChecked()



import sysconfig

NAME = "Notification"

DESCRIPTION = "Common tasks for notifications"

LONG_DESCRIPTION = "Common tasks for notifications"

ICON = "icons/category.svg"

BACKGROUND = "light-blue"

WIDGET_HELP_PATH = (
    # Development documentation (make htmlhelp in ./doc)
    ("{DEVELOP_ROOT}/doc/_build/htmlhelp/canvas/widgets/widgets.html", None),
    # Documentation included in wheel
    (
        "{}/help/ewoksnotify/canvas/widgets/widgets.html".format(
            sysconfig.get_path("data")
        ),
        None,
    ),
    # Online documentation url
    ("http://ewoksnotify.readthedocs.io/en/latest/canvas/widgets/widgets", ""),
)



from silx.gui import qt
from silx.gui.icons import getQIcon


class ParametersAction(qt.QAction):
    """
    Action to display a window with nxtomomill configuration
    """

    def __init__(self, parent):
        icon = getQIcon("ewoksnotify:gui/icons/parameters")

        qt.QAction.__init__(self, icon, "filter configuration", parent)
        self.setToolTip("Open dialog to configure nxtomomill parameters")
        self.setCheckable(False)



from orangewidget import gui
from silx.gui import qt
from ewoksnotify.gui.tone import ToneWindow
from ewoksorange.bindings.owwidgets import OWEwoksWidgetNoThread
from ewoksnotify.tasks.tone import ToneTask


class ToneNotifierOW(
    OWEwoksWidgetNoThread,
    ewokstaskclass=ToneTask,  # type: ignore
):
    """
    simple widget which pop up, ring and closes when recive a new object
    """

    name = "tone notifier"
    id = "orangecontrib.ewoksnotify.ToneNotifierOW"
    description = "Simple widget which pop up for 2 second when recives a new object and emit a tone"
    icon = "icons/notification.png"
    priority = 145
    keywords = ["notifier", "notification", "ewoks", "ewoksnotify"]

    _ewoks_inputs_to_hide_from_orange = (
        "tone_type",
        "theme",
        "muted",
        "with_text_message",
    )

    want_main_area = True
    want_control_area = False
    resizing_enabled = False

    def __init__(self, parent=None):
        super().__init__(parent)
        self.pop_up = None
        layout = gui.vBox(self.mainArea, self.name).layout()
        # define tone
        self._toneWidget = ToneWindow(parent=self)
        self._toneWidget.setWindowFlag(qt.Qt.Widget)
        layout.addWidget(self._toneWidget)
        # define pop up
        self._popUpCheckBox = qt.QCheckBox("trigger pop up", self)
        layout.addWidget(self._popUpCheckBox)

    def handleNewSignals(self) -> None:
        """Invoked by the workflow signal propagation manager after all
        signals handlers have been called.
        """
        my_object = super().get_task_inputs().get("object", None)
        if my_object is not None and self._popUpCheckBox.isChecked():
            self._activePopUp(my_object=my_object)
        super().handleNewSignals()

    def get_task_inputs(self):
        task_inputs = super().get_task_inputs()
        # when we are with the gui we go for the Qt pop up instead of the message to stdout
        task_inputs["with_text_message"] = False
        task_inputs.update(self._toneWidget.getConfiguration())
        return task_inputs

    def _activePopUp(self, my_object):
        if self.pop_up is not None:
            self.pop_up.close()

        self.pop_up = NotificationMessage()
        text = f"{my_object} received."
        self.pop_up.setText(text)
        self.pop_up.show()


class NotificationMessage(qt.QMessageBox):
    EXPOSITION_TIME = 3000  # in ms

    def __init__(self) -> None:
        super().__init__()
        self.setModal(False)
        self.setIcon(qt.QMessageBox.Information)
        self.addButton(
            f"Ok - will close automatically after {self.EXPOSITION_TIME / 1000}s",
            qt.QMessageBox.YesRole,
        )

    def show(self):
        super().show()
        timer = qt.QTimer(self)
        timer.singleShot(
            self.EXPOSITION_TIME,
            self.close,
        )



from ewoksnotify.tasks.waiter import WaiterTask


def test_WaiterTask():
    task = WaiterTask(
        inputs={
            "object": "toto",
            "seconds": 0.2,
        }
    )
    task.run()
    assert task.outputs.object == "toto"
